package fr.hibootracker;

import org.springframework.core.io.Resource;
import org.springframework.core.io.UrlResource;
import org.springframework.http.ResponseEntity;
import org.springframework.ui.Model;
import org.springframework.util.FileSystemUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.method.annotation.MvcUriComponentsBuilder;
import org.apache.commons.io.FilenameUtils;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * Created by Thomas on 07/02/2017.
 */

@RestController
@RequestMapping("/api")
public class StorageController {

    private final String pathStorage = "storage";
    Path rootLocation = Paths.get(pathStorage);

    @GetMapping("/{fileName:.+}")
    public byte[] doGet(@PathVariable String fileName) {
            Path path = Paths.get(pathStorage + "\\" + fileName);
            System.out.println("GET " + path.toString());
            try {
                byte[] data = Files.readAllBytes(path);
                return data;
            } catch (IOException e) {
                System.out.println("IOException: " + e.getMessage());
            }
        return null;
    }

    @GetMapping("/test")
    public String test() {

        return "Yann le doupé";
    }

    @GetMapping("/")
    public String listUploadedFiles(Model model) throws IOException {

        model.addAttribute("files",
                loadAll().map(path ->
                        MvcUriComponentsBuilder
                                .fromMethodName(StorageController.class, "serveFile", path.getFileName().toString())
                                .build().toString())
                .collect(Collectors.toList()));

        return "uploadForm";
    }

    @PostMapping(value = "/")
    public String handleFileUpload(@RequestParam("file") MultipartFile file) {

        store(file);

        return "done";
    }

    @ExceptionHandler(StorageFileNotFoundException.class)
    public ResponseEntity handleStorageFileNotFound(StorageFileNotFoundException exc) {
        return ResponseEntity.notFound().build();
    }

    public void store(MultipartFile file) {
        try {
            if (file.isEmpty()) {
                throw new StorageException("Failed to store empty file " + file.getOriginalFilename());
            }
            File coucou = new File(pathStorage + "\\" + file.getOriginalFilename());;
            while (coucou.exists()) {
                coucou = new File(pathStorage + "\\" + FilenameUtils.removeExtension(coucou.getName()) + "new." + FilenameUtils.getExtension(coucou.getName()));
            }
            Files.copy(file.getInputStream(), this.rootLocation.resolve(coucou.getName()));
        } catch (IOException e) {
            throw new StorageException("Failed to store file " + file.getOriginalFilename(), e);
        }
    }

    public Stream<Path> loadAll() {

        try {
            return Files.walk(rootLocation, 1)
                    .filter(path -> !path.equals(rootLocation))
                    .map(path -> rootLocation.relativize(path));
        } catch (IOException e) {
            throw new StorageException("Failed to read stored files", e);
        }

    }

    public Path load(String filename) {
        return rootLocation.resolve(filename);
    }

    public Resource loadAsResource(String filename) {
        try {
            Path file = load(filename);
            Resource resource = new UrlResource(file.toUri());
            if(resource.exists() || resource.isReadable()) {
                return resource;
            }
            else {
                throw new StorageFileNotFoundException("Could not read file: " + filename);
            }
        } catch (MalformedURLException e) {
            throw new StorageFileNotFoundException("Could not read file: " + filename, e);
        }
    }

    public void deleteAll() {
        FileSystemUtils.deleteRecursively(rootLocation.toFile());
    }

    public void init() {
        try {
            Files.createDirectory(rootLocation);
        } catch (IOException e) {
            throw new StorageException("Could not initialize storage", e);
        }
    }
}
